/* 
	RFCOMMd - RFCOMM daemon.
	Copyright (C) 2001 Qualcomm Incorporated

	Written 2001 by Maxim Krasnyansky <maxk@qualcomm.com>

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License version 2 as
	published by the Free Software Foundation;

	THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
	OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
	FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT OF THIRD PARTY RIGHTS.
	IN NO EVENT SHALL THE COPYRIGHT HOLDER(S) AND AUTHOR(S) BE LIABLE FOR ANY CLAIM,
	OR ANY SPECIAL INDIRECT OR CONSEQUENTIAL DAMAGES, OR ANY DAMAGES WHATSOEVER
	RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT,
	NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE
	USE OR PERFORMANCE OF THIS SOFTWARE.

	ALL LIABILITY, INCLUDING LIABILITY FOR INFRINGEMENT OF ANY PATENTS, COPYRIGHTS,
	TRADEMARKS OR OTHER RIGHTS, RELATING TO USE OF THIS SOFTWARE IS DISCLAIMED.
*/

/* 
	Based on:
	VTun - Virtual Tunnel over TCP/IP network.
	Copyright (C) 1998-2001 Maxim Krasnyansky <max_mk@yahoo.com>
*/

/*
 * $Id$
 */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include <signal.h>
#include <fcntl.h>
#include <syslog.h>
#include <sys/socket.h>

#include <bluetooth/bluetooth.h>
#include <bluetooth/l2cap.h>

#include "rfcommd.h"
#include "lib.h"
#include "llist.h"
#include "netlib.h"

static int terminate;
static void sig_term(int sig)
{
	syslog(LOG_INFO,"Terminated");
	terminate = -1;
}

void client(struct session *session)
{
	struct sockaddr_l2 my_addr,svr_addr;
	struct sigaction sa;
	int s, opt, reconnect;
	struct l2cap_options opts;

	syslog(LOG_INFO,"RFCOMMd client ver %s started", RFCOMMD_VER);

	memset(&sa,0,sizeof(sa));
	sa.sa_handler = SIG_IGN;
	sa.sa_flags = SA_NOCLDWAIT;
	sigaction(SIGHUP, &sa, NULL);
	sigaction(SIGINT, &sa, NULL);
	sigaction(SIGQUIT, &sa, NULL);
	sigaction(SIGPIPE, &sa, NULL);
	sigaction(SIGCHLD, &sa, NULL);

	sa.sa_handler = sig_term;
	sigaction(SIGTERM, &sa, NULL);

	terminate = 0; reconnect = 0;
	while( !terminate ){
		if( reconnect ) {
			if( session->flags & RFCOMMD_PERSIST ) {
				/* Persist mode. Sleep and reconnect. */
				sleep(5);
			} else {
				/* Exit */
				break;
			}
		} else {
			reconnect = 1;
		}

		io_init();

		set_title("%s init initializing", session->name);

		/* Set server address */
		if( server_addr(&svr_addr, session) < 0 )
			continue;

		/* Set local address */
		if( local_addr(&my_addr, session) < 0 )
			continue;

		if( (s = socket(AF_BLUETOOTH, SOCK_SEQPACKET, BTPROTO_L2CAP))==-1 ) {
			syslog(LOG_ERR,"Can't create socket. %s(%d)", strerror(errno), errno);
			continue;
		}

		/* Get default options */
		opt = sizeof(opts);
		if( getsockopt(s, SOL_L2CAP, L2CAP_OPTIONS, &opts, &opt) < 0 ) {
			syslog(LOG_ERR, "Can't get default L2CAP options. %s(%d)", strerror(errno), errno);
			continue;
		}

		opt=1;
		setsockopt(s, SOL_SOCKET, SO_REUSEADDR, &opt, sizeof(opt));

		if( bind(s, (struct sockaddr *)&my_addr, sizeof(my_addr)) ){
			syslog(LOG_ERR,"Can't bind socket. %s(%d)", strerror(errno), errno);
			continue;
		}

		set_title("connecting to %s", rfcommd.svr_addr);
		syslog(LOG_INFO,"Connecting to %s", rfcommd.svr_addr);

		if( !connect_t(s, (struct sockaddr *) &svr_addr, session->timeout) ){
			session->sock = s;
			terminate = start_session(session);
		} else
			syslog(LOG_INFO,"Connect to %s failed. %s(%d)", rfcommd.svr_addr, strerror(errno), errno);

		close(s);
		free_sopt(&session->sopt);
	}

	syslog(LOG_INFO, "Exit");
	return;
}
