/* 
	RFCOMMd - RFCOMM daemon.
	Copyright (C) 2001 Qualcomm Incorporated

	Written 2001 by Maxim Krasnyansky <maxk@qualcomm.com>

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License version 2 as
	published by the Free Software Foundation;

	THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
	OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
	FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT OF THIRD PARTY RIGHTS.
	IN NO EVENT SHALL THE COPYRIGHT HOLDER(S) AND AUTHOR(S) BE LIABLE FOR ANY CLAIM,
	OR ANY SPECIAL INDIRECT OR CONSEQUENTIAL DAMAGES, OR ANY DAMAGES WHATSOEVER
	RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT,
	NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE
	USE OR PERFORMANCE OF THIS SOFTWARE.

	ALL LIABILITY, INCLUDING LIABILITY FOR INFRINGEMENT OF ANY PATENTS, COPYRIGHTS,
	TRADEMARKS OR OTHER RIGHTS, RELATING TO USE OF THIS SOFTWARE IS DISCLAIMED.
*/

/* 
	Based on:
	VTun - Virtual Tunnel over TCP/IP network.
	Copyright (C) 1998-2001 Maxim Krasnyansky <max_mk@yahoo.com>
*/

/*
 * $Id$
 */

#include <stdlib.h>
#include "llist.h"

/* Function to work with the Linked Lists */

void llist_init(llist *l)
{
	l->head = l->tail = NULL;
}

int llist_empty(llist *l)
{
	return l->tail == NULL;
}

int llist_add(llist *l, void * d)
{
	llist_elm *e;

	if( !(e=malloc(sizeof(llist_elm))) )
		return -1;

	if( !l->head )
		l->head = l->tail = e;
	else
		l->tail->next = e;
	l->tail = e;

	e->next = NULL;
	e->data = d;

	return 0;
}

/* Travel list from head to tail */
void * llist_trav(llist *l, int (*f)(void *d, void *u), void *u)
{
	llist_elm *i = l->head;

	while( i ) {
		if( f(i->data,u) ) return i->data;
		i = i->next;
	}
	return NULL;
}

/* Copy list from (l) to (t) */
int llist_copy(llist *l, llist *t, void* (*f)(void *d, void *u), void *u)
{
	llist_elm *i = l->head;

	llist_init(t);

	while( i ) {
		llist_add(t,f(i->data,u));
		i = i->next;
	}
	return 0;
}

/* Travel list from head to tail, deallocate each element */
void * llist_free(llist *l, int (*f)(void *d, void *u), void *u)
{
	llist_elm *i = l->head, *n;
	void *ff = NULL;

	while( i ) {
		n = i->next;
		if( f(i->data,u) )
			ff = i->data;
		else
			free(i);
		i = n;
	}
	l->head = l->tail = NULL;
	return ff;
}


