/* 
	RFCOMMd - RFCOMM daemon.
	Copyright (C) 2001 Qualcomm Incorporated

	Written 2001 by Maxim Krasnyansky <maxk@qualcomm.com>

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License version 2 as
	published by the Free Software Foundation;

	THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
	OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
	FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT OF THIRD PARTY RIGHTS.
	IN NO EVENT SHALL THE COPYRIGHT HOLDER(S) AND AUTHOR(S) BE LIABLE FOR ANY CLAIM,
	OR ANY SPECIAL INDIRECT OR CONSEQUENTIAL DAMAGES, OR ANY DAMAGES WHATSOEVER
	RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT,
	NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE
	USE OR PERFORMANCE OF THIS SOFTWARE.

	ALL LIABILITY, INCLUDING LIABILITY FOR INFRINGEMENT OF ANY PATENTS, COPYRIGHTS,
	TRADEMARKS OR OTHER RIGHTS, RELATING TO USE OF THIS SOFTWARE IS DISCLAIMED.
*/

/* 
	Based on:
	VTun - Virtual Tunnel over TCP/IP network.
	Copyright (C) 1998-2001 Maxim Krasnyansky <max_mk@yahoo.com>
*/

/*
 * $Id$
 */ 

#ifndef _RFCOMMD_H
#define _RFCOMMD_H

#include <stdio.h>
#include "llist.h"

/* Default RFCOMMD connect timeout in sec */
#define RFCOMMD_CONNECT_TIMEOUT 30

/* General RFCOMMD timeout for several operations, in sec */
#define RFCOMMD_TIMEOUT 30

/* Number of seconds for delay after pppd startup */
#define RFCOMMD_DELAY_SEC  10 

/* Statistic interval in seconds */
#define RFCOMMD_STAT_IVAL  5*60  /* 5 min */

/* Max lenght of device name */
#define RFCOMMD_DEV_LEN  20 
 
/* #define DEBUG_EXEC 1 */

#if DEBUG_EXEC - 0
#define DEBUG_LOG(fmt, args...)  syslog(LOG_INFO, "%s (%d): " fmt, \
				__FUNCTION__, __LINE__ , ##args)
#else
#define DEBUG_LOG(fmt, args...)  do { } while (0)
#endif

#define DEBUG_MARK()  DEBUG_LOG("debug mark")

/* End of configurable part */

struct subst {
    char *dev;
    char *laddr;
    int  lpsm;
    char *raddr;
    int  rpsm;
};

struct stats {
   unsigned long byte_in;
   unsigned long byte_out;
   unsigned long comp_in;
   unsigned long comp_out;
   FILE *file;
};

struct addr {
   char *name;
   char *bdaddr;
   int psm;
   int type;
};
/* Address types */
#define RFCOMMD_ADDR_IFACE	0x01
#define RFCOMMD_ADDR_NAME  	0x02

#define RFCOMMD_STAT	   0x1000
#define RFCOMMD_PERSIST    0x2000

/* Support for multiple connections */
#define RFCOMMD_MULTI_DENY	0  /* no */ 
#define RFCOMMD_MULTI_ALLOW	1  /* yes */
#define RFCOMMD_MULTI_KILL	2

struct options {
   int  timeout;

   char *cfg_file;

   char *shell; 	 /* Shell */
   char *ppp;		 /* Command to configure ppp devices */
   char *ifcfg;		 /* Command to configure net devices */
   char *route;		 /* Command to configure routing */
   char *fwall; 	 /* Command to configure FireWall */

   char *svr_addr;       /* Server's address */
   int  svr_psm;	 /* Server's psm */
};

extern struct options rfcommd;

/* Session */
struct session {
   char *name;

   int  channel;
   int  flags;
   int  multi;
   int  timeout;
   int  persist;

   int  sock;
   int  pty;

   int  connected;
   int  handle; 

   char tty[50];

   llist up;
   llist down;

   struct addr  src_addr;
   struct stats stat;
   struct subst sopt;
};

extern llist sessions;

struct session * get_session_by_name(char *name);
struct session * get_session_by_channel(int channel);

void server(void);
void client(struct session *s);
int  read_config(char *file);

#endif
