/* 
	RFCOMMd - RFCOMM daemon.
	Copyright (C) 2001 Qualcomm Incorporated

	Written 2001 by Maxim Krasnyansky <maxk@qualcomm.com>

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License version 2 as
	published by the Free Software Foundation;

	THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
	OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
	FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT OF THIRD PARTY RIGHTS.
	IN NO EVENT SHALL THE COPYRIGHT HOLDER(S) AND AUTHOR(S) BE LIABLE FOR ANY CLAIM,
	OR ANY SPECIAL INDIRECT OR CONSEQUENTIAL DAMAGES, OR ANY DAMAGES WHATSOEVER
	RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT,
	NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE
	USE OR PERFORMANCE OF THIS SOFTWARE.

	ALL LIABILITY, INCLUDING LIABILITY FOR INFRINGEMENT OF ANY PATENTS, COPYRIGHTS,
	TRADEMARKS OR OTHER RIGHTS, RELATING TO USE OF THIS SOFTWARE IS DISCLAIMED.
*/

/* 
	Based on:
	VTun - Virtual Tunnel over TCP/IP network.
	Copyright (C) 1998-2001 Maxim Krasnyansky <max_mk@yahoo.com>
*/

/*
 * $Id$
 */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include <signal.h>
#include <fcntl.h>
#include <syslog.h>
#include <sys/socket.h>

#include <bluetooth/bluetooth.h>
#include <bluetooth/l2cap.h>

#include "rfcommd.h"
#include "lib.h"
#include "lock.h"

static void connection(int sock)
{
	struct sockaddr_l2 my_addr, cl_addr;
	struct sigaction sa;
	struct session s;
	bdaddr_t bdaddr;
	int opt;

	opt = sizeof(struct sockaddr_l2);
	if( getpeername(sock, (struct sockaddr *) &cl_addr, &opt) ){
		syslog(LOG_ERR, "Can't get peer name");
		exit(1);
	}
	opt = sizeof(struct sockaddr_l2);
	if( getsockname(sock, (struct sockaddr *) &my_addr, &opt) < 0 ){
		syslog(LOG_ERR, "Can't get local socket address");
		exit(1);
	}

	memset(&s, 0, sizeof(s));
	s.sock = sock;

	baswap(&bdaddr, &cl_addr.l2_bdaddr);
	s.sopt.raddr = strdup(batostr(&bdaddr));
	s.sopt.rpsm  = btohs(cl_addr.l2_psm);

	baswap(&bdaddr, &my_addr.l2_bdaddr);
	s.sopt.laddr = strdup(batostr(&bdaddr));
	s.sopt.lpsm  = rfcommd.svr_psm;

	memset(&sa, 0, sizeof(sa));
	sa.sa_handler = SIG_IGN;
	sa.sa_flags  = SA_NOCLDWAIT;
	sigaction(SIGHUP, &sa, NULL);

	syslog(LOG_INFO, "Connection from %s", s.sopt.raddr);

	start_session(&s);

	close(sock);

	syslog(LOG_INFO, "Exit");
	exit(0);
}

static void listener(void)
{
	struct sockaddr_l2 my_addr, cl_addr;
	struct l2cap_options opts;
	int s, s1, opt;

	memset(&my_addr, 0, sizeof(my_addr));
	bacpy(&my_addr.l2_bdaddr, BDADDR_ANY);
	my_addr.l2_family = AF_BLUETOOTH;
	my_addr.l2_psm    = htobs(rfcommd.svr_psm);

	if( (s=socket(AF_BLUETOOTH, SOCK_SEQPACKET, 0))== -1 ) {
		syslog(LOG_ERR,"Can't create socket");
		exit(1);
	}

	opt=1;
	setsockopt(s, SOL_SOCKET, SO_REUSEADDR, &opt, sizeof(opt));

	/* Get default options */
	opt = sizeof(opts);
	if( getsockopt(s, SOL_L2CAP, L2CAP_OPTIONS, &opts, &opt) < 0 ) {
		syslog(LOG_ERR, "Can't get default L2CAP options. %s(%d)", strerror(errno), errno);
		exit(1);
	}

	if( bind(s,(struct sockaddr *)&my_addr, sizeof(my_addr)) ) {
		syslog(LOG_ERR,"Can't bind socket");
		exit(1);
	}

	if( listen(s, 10) ){
		syslog(LOG_ERR,"Can't listen on the socket");
		exit(1);
	}

	set_title("waiting for connections on psm %d", rfcommd.svr_psm);

	while(1){
		opt=sizeof(cl_addr);
		if( (s1 = accept(s,(struct sockaddr *)&cl_addr,&opt)) == -1 )
			continue;

		switch( fork() ) {
			case 0:
				connection(s1);
				break;
			case -1:
				syslog(LOG_ERR, "Couldn't fork()");
			default:
				close(s1);
				break;
		}
	}
}

void server(void)
{
	syslog(LOG_INFO,"RFCOMMd server ver %s", RFCOMMD_VER);

	listener();
}
