/* 
	RFCOMMd - RFCOMM daemon.
	Copyright (C) 2001 Qualcomm Incorporated

	Written 2001 by Maxim Krasnyansky <maxk@qualcomm.com>

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License version 2 as
	published by the Free Software Foundation;

	THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
	OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
	FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT OF THIRD PARTY RIGHTS.
	IN NO EVENT SHALL THE COPYRIGHT HOLDER(S) AND AUTHOR(S) BE LIABLE FOR ANY CLAIM,
	OR ANY SPECIAL INDIRECT OR CONSEQUENTIAL DAMAGES, OR ANY DAMAGES WHATSOEVER
	RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT,
	NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE
	USE OR PERFORMANCE OF THIS SOFTWARE.

	ALL LIABILITY, INCLUDING LIABILITY FOR INFRINGEMENT OF ANY PATENTS, COPYRIGHTS,
	TRADEMARKS OR OTHER RIGHTS, RELATING TO USE OF THIS SOFTWARE IS DISCLAIMED.
*/

/* 
	Based on:
	VTun - Virtual Tunnel over TCP/IP network.
	Copyright (C) 1998-2001 Maxim Krasnyansky <max_mk@yahoo.com>
*/

/*
 * $Id$
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/time.h>
#include <unistd.h>
#include <string.h>
#include <errno.h>
#include <signal.h>

#include <netinet/in.h>
#include <arpa/inet.h>
#include <resolv.h>
#include <netdb.h>
#include <sys/socket.h>
#include <syslog.h>
#include <sys/time.h>
#include <termios.h>

#include <bluetooth.h>
#include <l2cap.h>

#include "rfcomm.h"
#include "rfcommd.h"
#include "lib.h"
#include "netlib.h"

int term;

int connect_ind(struct session *s, int channel);
int connect_cfm(struct session *s, int channel);

int event(rfcomm_iff *iff, int event, int channel)
{
	struct session *s = iff->priv;

	switch( event ){
		case CONNECT_IND:
			return connect_ind(s, channel);
		case CONNECT_CFM:
			return connect_cfm(s, channel);
	}
}

int start_session(struct session *s)
{
	rfcomm_iff iff;
	int status;

	if( (s->pty = pty_open(s->tty, sizeof(s->tty) - 1)) < 0 ){
		syslog(LOG_ERR, "Can't allocate pseudo tty. %s(%d)", strerror(errno), errno);
		return -1;
	}

	rfcomm_init();

	iff.sock  = s->sock;
	iff.pty   = s->pty;
	iff.event = event;
	iff.priv  = (void *)s;
	if( (s->handle = rfcomm_register_iface(&iff)) < 0 ){
		syslog(LOG_ERR, "Can't register RFCOMM interface\n");
		return 0;
	}

	/* Start RFCOMM */
	status = rfcomm_open(s->handle, s->channel);

	if( s->connected )
		llist_trav(&s->down, run_cmd, s);

	close(s->pty);
	close(s->sock);
	return status;
}

int connect_ind(struct session *s, int channel)
{
	struct session *ss;

	/* Reject connections if we're a client */
	if( s->channel )
		return 0;

	if( !(ss = get_session_by_channel(channel)) ){
		syslog(LOG_ERR,"No session for channel %d", channel);
		return 0;
	}

	/* Copy session paramenters */ 	
	s->name = ss->name;
	s->up   = ss->up;
	s->down = ss->down;

	return 1;
}

int connect_cfm(struct session *s, int channel)
{
	struct termios ti;
	int pid;

	switch( (pid = fork()) ){
		case -1:
			syslog(LOG_ERR, "Couldn't fork()");
			return 0;

		case 0:
			/* Child */

			/* Socket and PTY are not used in the child */
			close(s->sock);
			close(s->pty);
			{
			  int fd_in;

			  /* Route stdin to the pseudo tty */
			  close(0);
			  if( (fd_in = open(s->tty, O_RDWR)) < 0 ){
				syslog(LOG_ERR, "Couldn't open slave pty");
				exit(1);
			  }
			  dup2(fd_in, 0);
	
			  /* Route stdout to stdin, stderr to /dev/null */
			  close(1); dup2(fd_in, 1);
			  close(2); open("/dev/null", O_RDWR);
			}

			/* Switch tty to RAW mode */
			tcgetattr(0, &ti);
			cfmakeraw(&ti); 
			tcsetattr(0, TCSANOW, &ti);

			/* Run list of up commands */
			set_title("%s running up commands", s->name);
			llist_trav(&s->up, run_cmd, s);

			exit(0);
	}
	
	/* Parent */
	s->connected = 1;

	syslog(LOG_INFO,"Session %s[%s] opened", s->name, s->sopt.raddr);
	set_title("%s channel %d pid %d", s->name, channel, pid);

	return 1;
}
